@@ -655,6 +655,7 @@ static void osinfo_loader_install_config_params(OsinfoLoader *loader,
                 osinfo_install_config_param_set_value_map(param, map);
         }
 
+        xmlFree(mapid);
         xmlFree(name);
         xmlFree(policy);
     };
@@ -855,6 +856,7 @@ static OsinfoMedia *osinfo_loader_media (OsinfoLoader *loader,
                     osinfo_entity_set_param(OSINFO_ENTITY(media),
                                             OSINFO_MEDIA_PROP_LANG_MAP,
                                             datamap);
+                xmlFree(datamap);
             } else {
                 osinfo_entity_add_param(OSINFO_ENTITY(media),
                                         OSINFO_MEDIA_PROP_LANG,
@@ -1784,18 +1786,18 @@ void osinfo_loader_process_system_path(OsinfoLoader *loader,
 {
     GFile *file;
     gchar *dbdir;
-    const gchar *path = getenv("OSINFO_DATA_DIR");
+    const gchar *path = g_getenv("OSINFO_DATA_DIR");
     if (!path)
         path = PKG_DATA_DIR;
 
     dbdir = g_strdup_printf("%s/db", path);
-
     file = g_file_new_for_path(dbdir);
     osinfo_loader_process_file(loader,
                                file,
                                FALSE,
                                err);
     g_object_unref(file);
+    g_free(dbdir);
 }
 
 void osinfo_loader_process_local_path(OsinfoLoader *loader, GError **err)
